import time


class AccountError(Exception):
    """Custom exception for account-related errors."""
    pass

class transaction:
    def __init__(self, allUser):
        self.allUser = allUser

    def _validate_account(self, cardNum):
        user = self.allUser.get(cardNum)
        if not user:
            raise AccountError("This account does not exist.")
        if user.card.cardLock:
            raise AccountError("This account is locked down.")
        return user

    def _check_password(self, realPasswd):
        for i in range(3):
            tempPasswd = input("Please enter your password: ")
            if tempPasswd == realPasswd:
                return True
        raise AccountError("Wrong password. Your account is locked down...")

    def balance(self):
        try:
            cardNum = input("Please enter your account number: ")
            user = self._validate_account(cardNum)
            if not self._check_password(user.card.cardPasswd):
                user.card.cardLock = True
                return
            print("Account number：%s  Account balance：%d" % (user.card.cardId, user.card.cardMoney))
        except AccountError as e:
            print(e)

    def withdraw(self):
        try:
            cardNum = input("Please enter your account number: ")
            user = self._validate_account(cardNum)
            self._check_password(user.card.cardPasswd)
            
            money = int(input("Please enter withdraw amount："))
            if money > user.card.cardMoney or money < 0:
                raise AccountError("Invalid amount. Withdrawal failed.")

            user.card.cardMoney -= money
            print("Withdraw success. Account balance: %d" % user.card.cardMoney)
        except AccountError as e:
            print(e)
        except ValueError:
            print("Invalid input. Please enter a numeric value.")

    def deposit(self):
        try:
            cardNum = input("Please enter your account number: ")
            user = self._validate_account(cardNum)
            self._check_password(user.card.cardPasswd)
            
            money = int(input("Please enter deposit amount："))
            if money < 0:
                raise AccountError("Invalid amount. Deposit failed.")

            user.card.cardMoney += money
            print("Deposit success: %d. Account balance: %d" % (money, user.card.cardMoney))
        except AccountError as e:
            print(e)
        except ValueError:
            print("Invalid input. Please enter a numeric value.")

    def transfer(self):
        try:
            cardNum = input("Please enter your account number: ")
            user = self._validate_account(cardNum)
            self._check_password(user.card.cardPasswd)

            money = int(input("Please enter transfer amount："))
            if money > user.card.cardMoney or money < 0:
                raise AccountError("Invalid amount. Transfer failed.")

            newcardNum = input("Please enter the recipient account number：")
            newcard = self._validate_account(newcardNum)

            user.card.cardMoney -= money
            newcard.card.cardMoney += money
            time.sleep(1)
            print("Transfer success")
            time.sleep(1)
            print("Transfer amount: %d. Account balance: %d" % (money, user.card.cardMoney))
        except AccountError as e:
            print(e)
        except ValueError:
            print("Invalid input. Please enter a numeric value.")

    def checkPasswd(self, realPasswd):
        for i in range(3):
            tempPasswd = input("Please enter your password: ")
            if tempPasswd == realPasswd:
                return True
        return False


from .store import Store
from .card import User, Card
from .action import action

import time 

#inheritance class
class AdminView(Store):

    def __init__(self, run=True):
        super(AdminView, self).__init__()
        self.action = action(self.users)
        if run:
            self.admin_view()

    def quit(self):
        print('Bye~')
        self.save_all_data()



    def login(self):
        _user = input('admin username: ')
        _password = input('admin password: ')

        if _user == '1' and _password == '1':
            print('Login successful!')
            # Check if the user is already in the dictionary
            
            return True
        else:
            print('Incorrect username or password!')
            self.users = None
            return False

              
    
    def admin_view(self):

        if not self.login():
            print('Password is wrong，Bye~')
            return

        while True:
            print("*************************************************")
            print("*             Create User（1）                   *")
            print("*          Deactivate User（0）                  *")
            print("*                 Exit（q）                      *")
            print("*************************************************")
            option = input("Please enter your option:")
            if option == "1":  
                self.action.createUser()
            elif option == "0":
                self.action.killUser()
            elif option == "q":
                self.quit()
                break
            else:
                raise ValueError("Invalid option provided")
                


if __name__ == '__main__':
    AdminView()

    
    


from admin.view import AdminView
from user.account import UserView
import time


class MainView:
    def __init__(self):
        self.admin = AdminView
        self.user = UserView
        self.choice_user_type()

    def run_admin(self):
        self.admin(False).admin_view()

    def run_user(self):
        self.user(False).user_view()

    def choice_user_type(self):
        print("*************************************************")
        print("*                                               *")
        print("*                                               *")
        print("*                  Welcome                      *")
        print("*                                               *")
        print("*                                               *")
        print("*************************************************")
        time.sleep(0.5)

        print('=================================================')
        print('Please select your role：')
        print('1. admin')
        print('2. user')
        print('=================================================')
        user = input('>')
        if user not in ('1', '2'):
            return
        if user == '1':
            self.run_admin()
        else:
            self.run_user()

def start_app():
    MainView()

if __name__ == '__main__':
    start_app()


import unittest
from unittest.mock import patch, MagicMock
from user.account import UserView


class TestAccount(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("Set up class resources...")
        cls.transaction_patch = patch('user.transaction.transaction')
        cls.mock_transaction = cls.transaction_patch.start()
        cls.mock_transaction.return_value.withdraw = MagicMock(return_value=True)
        cls.mock_transaction.return_value.deposit = MagicMock(return_value=True)

    @classmethod
    def tearDownClass(cls):
        print("Cleaning up class resources...")
        cls.transaction_patch.stop()

    def setUp(self):
        self.user_view = UserView(run=False)

    def tearDown(self):
        self.user_view = None

    def testSelectWithdraw(self):
        with patch('builtins.input', side_effect=['2', '123456', '1234', '50', 'q']), \
             patch('builtins.print') as mock_print:
            self.user_view.user_view()
            self.assertFalse(TestAccount.mock_transaction.return_value.withdraw.called)
            self.assertIsNotNone(TestAccount.mock_transaction.return_value.withdraw)
            self.assertIsNotNone(self.user_view)
            self.assertIsInstance(TestAccount.mock_transaction.return_value, MagicMock)

    def testSelectDeposit(self):
        with patch('builtins.input', side_effect=['3', '123456', '1234', '50', 'q']), \
             patch('builtins.print') as mock_print:
            self.user_view.user_view()

            self.assertEqual(TestAccount.mock_transaction.return_value.deposit.call_count, 0)
            self.assertIsInstance(self.user_view, UserView)
            self.assertFalse(self.user_view is None)
            self.assertIs(self.user_view, self.user_view)


if __name__ == '__main__':
    unittest.main()



import unittest
from test_viewandaction import TestAdminView 
from test_action import TestAction
from test_account import TestAccount  
from test_transaction import TestTransaction
  

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestAdminView))
    suite.addTest(unittest.makeSuite(TestAction))
    suite.addTest(unittest.makeSuite(TestAccount))
    suite.addTest(unittest.makeSuite(TestTransaction))

    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())



import unittest
from admin.action import action 
from admin.card import Card, User
import time
import random
from unittest.mock import patch, Mock
from itertools import cycle

from admin.view import AdminView
from admin.store import Store


class TestAdminView(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        print("Set up class")
    
    @classmethod    
    def tearDownClass(cls):
        print("Tearing down the test class")
        
    def setUp(self):
        print("Setting up for a test case")
        self.allUser = {}
        self.action_instance = action(self.allUser)
        
        self.admin_view_instance = AdminView(run=False)
        
    def tearDown(self):
        print("Tearing down after a test case")
        self.allUser = None
        

    def test_createUser(self):
        with patch('builtins.input', side_effect=['John Doe', '123456789', '1234567890', '1000', '1234', '1234']):
            my_action = action(self.allUser)
            my_action.createUser()
            card_id = list(self.allUser.keys())[0]
            user = self.allUser[card_id]

            self.assertEqual(user.name, 'John Doe')
            self.assertEqual(user.idCard, '123456789')
            self.assertEqual(user.phone, '1234567890')
            self.assertEqual(user.card.cardMoney, 1000)
            self.assertEqual(user.card.cardPasswd, '1234')
            
    def test_checkPasswd(self):       
        real_password = '12345'

        with patch('builtins.input', return_value=real_password):
            result = self.action_instance.checkPasswd(real_password)
            self.assertTrue(result)

        # Test with incorrect password
        with patch('builtins.input', return_value='111111'):
            result = self.action_instance.checkPasswd(real_password)
            self.assertFalse(result)

        # Test with empty input
        with patch('builtins.input', return_value=''):
            result = self.action_instance.checkPasswd(real_password)
            self.assertFalse(result)

        # Test with multiple incorrect inputs followed by correct input
        with patch('builtins.input', side_effect=['wrong1', 'wrong2', 'wrong3', real_password]):
            result = self.action_instance.checkPasswd(real_password)
            self.assertFalse(result)
            
    def test_login_credentials(self):
        with patch('builtins.input', side_effect=['1', '1']):
            result = self.admin_view_instance.login()            
            self.assertTrue(result)           
            self.assertNotIn('1', self.admin_view_instance.users)
    
            
        with patch('builtins.input', side_effect=['1', '2']):
            result = self.admin_view_instance.login()            
            self.assertFalse(result)
            
        with patch('builtins.input', side_effect=['3', '2']):
            result = self.admin_view_instance.login()            
            self.assertFalse(result)
            
            
   
        

if __name__ == '__main__':
    unittest.main()


import unittest
from admin.action import action 
from admin.card import Card, User
import time
import random
from unittest.mock import patch


class TestAction(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        print("Set up class")
        
    def setUp(self):
        print("Setting up for a test case")
        self.allUser = {}
        
    def tearDown(self):
        print("Tearing down after a test case")
        self.allUser = None
        
    @classmethod    
    def tearDownClass(cls):
        print("Tearing down the test class")

    def test_randomCardId(self):
        my_action = action(self.allUser)
        card_id = my_action.randomCardId()
        duplicate_card_id = my_action.randomCardId()
        
        self.assertEqual(len(card_id), 12)
        self.assertNotIn(card_id, my_action.allUser)
        self.assertTrue(card_id.isdigit())
        self.assertNotEqual(card_id, duplicate_card_id)

            
    def test_killUser(self):
        card_id = '123456789012'  
        user_data = {'name': 'Test User', 'idCard': '987654321', 'phone': '9876543210', 'card': Card(card_id, '5678', 500)}
        self.allUser[card_id] = User(**user_data)
        
        non_existent_card_id = '1111111111'

        with patch('builtins.input', side_effect=[card_id, '5678']):
            my_action = action(self.allUser)
            my_action.killUser()

            self.assertNotIn(card_id, self.allUser)
            self.assertEqual(len(self.allUser), 0)
            self.assertNotIn(non_existent_card_id, self.allUser)
            
            
        with patch('builtins.input', side_effect=[non_existent_card_id, '5678']):
            my_action = action(self.allUser)
            result = my_action.killUser()
            self.assertEqual(result, -1)

if __name__ == '__main__':
    unittest.main()


from admin.store import Store
from user.transaction import transaction


class UserView(Store):
    def __init__(self, run=True):
        super(UserView, self).__init__()
        self.transaction = transaction(self.users)
        if run:
            self.user_view()

    def menu(self):
        print("*************************************************")
        print("*    Balance（1）                                *")
        print("*    Withdraw（2）                               *")
        print("*    Deposit（3）                                *")
        print("*    Transfer（4）                               *")
        print("*    Quit（q）                                   *")
        print("*************************************************")

    def user_view(self):
        while True:
            self.menu()
            option = input("Please enter the process number：")
            if option == '1':
                self.transaction.balance()
            elif option == '2':
                self.transaction.withdraw()
            elif option == '3':
                self.transaction.deposit()
            elif option == '4':
                self.transaction.transfer()
            elif option == 'q':
                self.quit()
                break
            else:
                print('Invalid option. Please enter again...')
   
    def quit(self):
        print('Quiting...')
        self.save_all_data()



if __name__ == '__main__':
    UserView()

    


import unittest
from unittest.mock import patch
from user.transaction import transaction
from admin.card import Card
from admin.card import User
import time

test_card = Card('123456', '1234', 100)
test_user = User('Test User', '1234567890', '555-5555', test_card)
test_users = {'123456': test_user}

class TestTransaction(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_card = Card('123456', '1234', 100)
        cls.test_user = User('Test User', '1234567890', '555-5555', cls.test_card)
        cls.test_users = {'123456': cls.test_user}
        print("Set up class resources...")

    @classmethod
    def tearDownClass(cls):
        print("Cleaning up class resources...")

    def setUp(self):
        self.allUser = TestTransaction.test_users.copy()
        self.transaction = transaction(self.allUser)

    def tearDown(self):
        TestTransaction.test_user.card.cardMoney = 100
        TestTransaction.test_user.card.cardLock = False
        self.allUser = None

    def testDeposit(self):
        with patch('builtins.input', side_effect=['123456', '1234', '-50']):
            self.transaction.deposit()
            self.assertEqual(TestTransaction.test_user.card.cardMoney, 100)
            self.assertIsNotNone(TestTransaction.test_user)
            self.assertIsNotNone(TestTransaction.test_user.card)
            self.assertFalse(TestTransaction.test_user.card.cardLock)
    
    def testWithdraw(self):
        with patch('builtins.input', side_effect=['123456', '1234', '50']):
            self.transaction.withdraw()
            self.assertEqual(TestTransaction.test_user.card.cardMoney, 50)
            self.assertIsNotNone(TestTransaction.test_user)
            self.assertIsNotNone(TestTransaction.test_user.card)
            self.assertFalse(TestTransaction.test_user.card.cardLock)

    def testTransfer(self):
        recipient_card = Card('654321', '4321', 200)
        recipient_user = User('Recipient User', '0987654321', '555-5556', recipient_card)
        self.allUser['654321'] = recipient_user
        
        with patch('builtins.input', side_effect=['123456', '1234', '50', '654321']):
            self.transaction.transfer()
            self.assertEqual(TestTransaction.test_user.card.cardMoney, 50)
            self.assertEqual(recipient_user.card.cardMoney, 250)
            self.assertIsNotNone(recipient_user)
            self.assertIsNotNone(recipient_user.card)
            self.assertFalse(recipient_user.card.cardLock)

    def testBalance_cardlocked(self):
        TestTransaction.test_user.card.cardLock = True
        with patch('builtins.input', side_effect=['123456']):
            balance = self.transaction.balance()
            self.assertTrue(TestTransaction.test_user.card.cardLock)
            self.assertIsNotNone(TestTransaction.test_user.card.cardId)
            self.assertIsNotNone(TestTransaction.test_user.card)

    def testBalance_incorpasswrd(self):
        with patch('builtins.input', side_effect=['123456', 'wrong_password', 'wrong_password', 'wrong_password']):
            balance = self.transaction.balance()
            self.assertIsNotNone(TestTransaction.test_user.card.cardId)
            self.assertIsNotNone(TestTransaction.test_user.card)

    @patch('builtins.input', side_effect=['123456', '1234', '50', 'nonexistent'])
    def testTransfer_nonextAcct(self, mock_inputs):
        self.transaction.transfer()
        self.assertEqual(TestTransaction.test_user.card.cardMoney, 100)

    @patch('builtins.input', side_effect=['123456', '1234', '50', '654321'])
    def testTransfer_locked(self, mock_inputs):
        recipient_card = Card('654321', '4321', 200)
        recipient_user = User('Recipient User', '0987654321', '555-5556', recipient_card)
        recipient_user.card.cardLock = True
        TestTransaction.test_users['654321'] = recipient_user
        self.transaction.transfer()
        self.assertEqual(TestTransaction.test_user.card.cardMoney, 100)
        
    @patch('builtins.input', side_effect=['123456', '1234', '200'])
    def testWithdraw_excbal(self, mock_inputs):
        self.transaction.withdraw()
        self.assertEqual(TestTransaction.test_user.card.cardMoney, 100)

    @patch('builtins.input', side_effect=['123456', '1234', '-50'])
    def testWithdraw_neg(self, mock_inputs):
        self.transaction.withdraw()
        self.assertEqual(TestTransaction.test_user.card.cardMoney, 100)   

if __name__ == '__main__':
    unittest.main()


class Card(object):
    def __init__(self, cardId, cardPasswd, cardMoney, cardLock=False):
        self.cardId = cardId
        self.cardPasswd = cardPasswd
        self.cardMoney = cardMoney
        self.cardLock = cardLock
      

    def to_dict(self):
        return {
            'cardId': self.cardId,
            'cardPasswd': self.cardPasswd,
            'cardMoney': self.cardMoney
        }


class User(object):
    def __init__(self, name, idCard, phone, card):
        self.name = name
        self.idCard = idCard
        self.phone = phone
        self.card = card

    def to_dict(self):
        return {
            'name': self.name,
            'idCard': self.idCard,
            'phone': self.phone,
            'card': self.card.to_dict()
        }


import os
import json
from .action import action
from .card import Card, User

current_work_dir = os.path.dirname(__file__)


class Store():
    data_file_path = os.path.join(current_work_dir, '../data.json')

    def __init__(self):
        self.users = self.read_users_from_file()
        

    def save_all_data(self):
        new_data = {}
        for k in self.users.keys():
            new_data[k] = self.users[k].to_dict()
        with open(self.data_file_path, 'w', encoding='utf-8') as f:
            f.write(json.dumps(new_data, ensure_ascii=False))

    def read_users_from_file(self):
        with open(self.data_file_path, 'r', encoding='utf-8') as f:
            data = json.loads(f.read())
            for k, v in data.items():
                v['card'] = Card(**v['card'])
                data[k] = User(**v)
            return data

        
    def admin_view(self):
        pass

    def user_view(self):
        pass



import time
import random
from .card import Card, User

class action(object):
    
    def __init__(self, allUser):
        self.allUser = allUser
        
   
    def createUser(self):
        try:
            name = input("Please enter your name:")
            idCard = input("Please enter your id: ")
            phone = input("Please enter your phone number")
            prestoreMoney = int(input("Please enter your prestore money:"))

            if prestoreMoney <= 0:
                raise ValueError("Prestore money must be greater than 0")

            firstPasswd = input("Please enter password:")
            secondPasswd = input("Please enter password again:")

            if firstPasswd != secondPasswd:
                raise ValueError("Passwords do not match")

            print("Password set, please remember it: %s" % secondPasswd)

            cardId = self.randomCardId()
            card = Card(cardId, secondPasswd, prestoreMoney)
            user = User(name, idCard, phone, card)

            # Save
            self.allUser[cardId] = user
            print("Create User Successfully! Please remember the card number...(%s)" % cardId)

        except ValueError as e:
            print("Error: {}".format(e))
            return -1


        
    
    def randomCardId(self):
        while True:
            str = ""
            for i in range(12):
                ch = chr(random.randrange(ord('0'), ord('9') + 1))
                str += ch
            if not self.allUser.get(str):
                return str
            
        
    def killUser(self):
        try:
            cardNum = input("Please enter card number:")

            if not cardNum.isnumeric():
                raise ValueError("Card number must be numeric")

            user = self.allUser.get(cardNum)
            if not user:
                raise ValueError("User does not exist")

            # Check password
            if not self.checkPasswd(user.card.cardPasswd):
                raise ValueError("Incorrect password")

            del self.allUser[cardNum]
            time.sleep(1)
            print("Remove the user successfully!")

        except ValueError as e:
            print("Error: {}".format(e))
            return -1
        

    def checkPasswd(self, realPasswd):
        try:
            tempPasswd = input("Please enter password: ")
            if tempPasswd == realPasswd:
                return True
            else:
                raise ValueError("Incorrect Password")
        except ValueError as e:
            
            print("Error: {}".format(e))
            return False
        except Exception as e:
            print("An unexpected error occurred: {}".format(e))
            return False
